import { Op, Sequelize } from "sequelize";

import Db from "..";
import GenericExtr from "../../db/extr/generic-extr";

export default class ExtrDb {
  private _db: Db;
  private _tableName: string;

  constructor(db: Db, tableName: string) {
    this._db = db;
    this._tableName = tableName;

    this._db.defineModel(this._tableName);
  }

  private static getStartEndDay(date: string): { start: Date, end: Date } {
    const parsedDate = new Date(date);
    const start = new Date(Date.UTC(parsedDate.getUTCFullYear(), parsedDate.getUTCMonth(), parsedDate.getUTCDate(), 0, 0, 0, 0));
    const end = new Date(Date.UTC(parsedDate.getUTCFullYear(), parsedDate.getUTCMonth(), parsedDate.getUTCDate(), 23, 59, 59, 999));

    return { start, end };
  }

  public async getByCode(code: string, requestedFields: Array<string>): Promise<GenericExtr | null> {
    const result = await this._db.models()[this._tableName].findOne({ attributes: requestedFields, where: { code } });
    if (!result) {
      return null;
    }

    return { code, ...result.get() };
  }

  public async getAll(requestedFields: Array<string>): Promise<Array<any>> {
    const result = await this._db.models()[this._tableName].findAll({ attributes: requestedFields });
    if (!result) {
      return [];
    }

    return result;
  }

  public async getFilteredBy(filters: any, requestedFields: Array<string>): Promise<Array<any>> {
    const where: any = { };

    if (filters.year && filters.year >= 1900) {
      where.year = {
        [Op.eq]: filters.year
      };
    }

    if (filters.maker) {
      where.maker = {
        [Op.like]: `%${filters.maker}%`
      };
    }

    if (filters.model) {
      where.model = {
        [Op.like]: `%${filters.model}%`
      };
    }

    if (filters.version) {
      where.version = {
        [Op.like]: `%${filters.version}%`
      };
    }

    if (filters.color) {
      where.color = {
        [Op.like]: `%${filters.color}%`
      };
    }

    if (filters.price && filters.price >= 0) {
      where.price = {
        [Op.eq]: filters.price
      };
    }

    if (filters.code) {
      where.code = {
        [Op.eq]: filters.code
      };
    }

    if (filters.title) {
      where.title = {
        [Op.like]: `%${filters.title}%`
      };
    }
    
    if (filters.description) {
      where.description = {
        [Op.like]: `%${filters.description}%`
      };
    }
    
    if (filters.plate) {
      where.plate = {
        [Op.eq]: filters.plate
      };
    }
    
    if (filters.mileage && filters.mileage >= 0) {
      where.mileage = {
        [Op.eq]: filters.mileage
      };
    }
    
    if (filters.gearbox) {
      where.gearbox = {
        [Op.like]: `%${filters.gearbox}%`
      };
    }
    
    if (filters.engine) {
      where.engine = {
        [Op.like]: `%${filters.engine}%`
      };
    }
    
    if (filters.fuel) {
      where.fuel = {
        [Op.like]: `%${filters.fuel}%`
      };
    }
    
    if (filters.imageUrl) {
      where.imageUrl = {
        [Op.eq]: filters.imageUrl
      };
    }
    
    if (filters.code) {
      where.code = {
        [Op.eq]: filters.code
      };
    }
    
    if (filters.url) {
      where.url = {
        [Op.eq]: filters.url
      };
    }
    
    if (filters.region) {
      where.region = {
        [Op.like]: `%${filters.region}%`
      };
    }
    
    if (filters.city) {
      where.city = {
        [Op.like]: `%${filters.city}%`
      };
    }
    
    if (filters.sellerType) {
      where.sellerType = {
        [Op.like]: `%${filters.sellerType}%`
      };
    }
    
    if (filters.sellerName) {
      where.sellerName = {
        [Op.like]: `%${filters.sellerName}%`
      };
    }
    
    if (filters.publicationCreationDate) {
      const { start, end } = ExtrDb.getStartEndDay(filters.publicationCreationDate);
      where.publicationCreationDate = {
        [Op.between]: [
          Sequelize.fn('STR_TO_DATE', start, '%Y-%m-%d %H:%i:%s'),  // Ajusta el formato según sea necesario
          Sequelize.fn('STR_TO_DATE', end, '%Y-%m-%d %H:%i:%s')
        ]
      };
    }

    if (filters.publicationCreationRawDate) {
      const { start, end } = ExtrDb.getStartEndDay(filters.publicationCreationRawDate);
      where.publicationCreationRawDate = {
        [Op.between]: [
          Sequelize.fn('STR_TO_DATE', start, '%Y-%m-%d %H:%i:%s'),  // Ajusta el formato según sea necesario
          Sequelize.fn('STR_TO_DATE', end, '%Y-%m-%d %H:%i:%s')
        ]
      };
    }
    
    if (filters.publicationUpdateDate) {
      const { start, end } = ExtrDb.getStartEndDay(filters.publicationUpdateDate);
      where.publicationUpdateDate = {
        [Op.between]: [
          Sequelize.fn('STR_TO_DATE', start, '%Y-%m-%d %H:%i:%s'),  // Ajusta el formato según sea necesario
          Sequelize.fn('STR_TO_DATE', end, '%Y-%m-%d %H:%i:%s')
        ]
      };
    }
    
    if (filters.publicationCreatedAt) {
      const { start, end } = ExtrDb.getStartEndDay(filters.publicationCreatedAt);
      where.publicationCreatedAt = {
        [Op.between]: [
          Sequelize.fn('STR_TO_DATE', start, '%Y-%m-%d %H:%i:%s'),  // Ajusta el formato según sea necesario
          Sequelize.fn('STR_TO_DATE', end, '%Y-%m-%d %H:%i:%s')
        ]
      };
    }

    const result = await this._db.models()[this._tableName].findAll({ attributes: requestedFields, where });
    if (!result) {
      return [];
    }

    return result;
  }
}
