import * as cors from "cors";
import * as express from "express";
import * as morgan from "morgan";
import * as helmet from "helmet";
import { ApolloServer } from "@apollo/server";
import { expressMiddleware } from "@apollo/server/express4";

import { apiRouter, webRouter } from "./routers/v1";
import { swaggerUIServe, swaggerUISetup } from "../../libs/swagger";
import { schema, root } from "../../libs/graphql";

export default class ExpressApp {
  private static _instance: express.Express;

  // eslint-disable-next-line @typescript-eslint/no-empty-function
  private constructor() {}

  private static initRouters() {
    this._instance.use("/api/v1", apiRouter);
    this._instance.use("/", webRouter);
    this._instance.use("/swagger", swaggerUIServe, swaggerUISetup);
  }

  private static async initApolloServer() {
    const apolloServer = new ApolloServer({
      typeDefs: schema,
      rootValue: root
    });
    await apolloServer.start();
    this._instance.use("/graphql", cors<cors.CorsRequest>(), express.json(), expressMiddleware(apolloServer));
  }

  public static get instance(): express.Express {
    if (!this._instance) {
      this._instance = express();
      this._instance.use(cors());
      this._instance.use(morgan("combined"));
      this._instance.use(express.urlencoded({ extended: true }));
      this._instance.use(express.json());

      // this._instance.use(helmet.contentSecurityPolicy());
      // this._instance.use(helmet.crossOriginEmbedderPolicy());
      this._instance.use(helmet.crossOriginOpenerPolicy());
      this._instance.use(helmet.crossOriginResourcePolicy());
      this._instance.use(helmet.dnsPrefetchControl());
      this._instance.use(helmet.frameguard());
      this._instance.use(helmet.hidePoweredBy());
      this._instance.use(helmet.hsts());
      this._instance.use(helmet.ieNoOpen());
      this._instance.use(helmet.noSniff());
      this._instance.use(helmet.originAgentCluster());
      this._instance.use(helmet.permittedCrossDomainPolicies());
      this._instance.use(helmet.referrerPolicy());
      this._instance.use(helmet.xssFilter());

      this.initRouters();
      this.initApolloServer();
    }
    return this._instance;
  }
}
